@@ -1,3 +1,10 @@
+"""
+Classes, that represents various value types that can appear in the configuration and problem definitionfiles.
+
+Each grammar type can both parse string containing a value of a given type, and to create the string containing a given value.
+"""
+
+
 import pyparsing as pp
 import io
 import inspect
@@ -15,14 +22,16 @@ from .grammar import generate_grammar, separator as separator_grammar, \
 from ase.units import Rydberg
 import copy
 import datetime
+from typing import Union, Any, Callable
 
 context =  generate_grammar()
 context.__enter__()
+#it ensures that the generated grammar will have the correct whitespaces
 
 class BaseType:
-  """ Base class for definition of configuration option types """
+  """ Base class for definition of configuration option types
 
-  """ The type without value (e.g. Separator) are just syntactical
+      A type without value (e.g. Separator) are just syntactical
       elements in the potentials file, that do not carry an information.
       Such elements do not yields (name, value) pair during parsing the file.
 
@@ -32,7 +41,8 @@ class BaseType:
       the Flag is False), but the name-value tuple of such Type is present
       in the parse result. On the other hand, has_value = False is in the file, but
       not in the result.
-  """
+
+        """
   has_value = True
 
   """ Default value for BaseValueDefinition.name_in_grammar.
@@ -41,7 +51,8 @@ class BaseType:
   """
   name_in_grammar = True
 
-  """ Default value for the given type. It can be overriden in the constructor. """
+  """ Default value for the given type. It can be overriden in the constructor (or just by setting
+  the instantiated object attribute) """
   default_value = None
 
   """ Deafault type for creating numpy arrays (e.g. by Table) is object - to be redefined
@@ -49,10 +60,46 @@ class BaseType:
   """
   numpy_type = object
 
-  def __init__(self, prefix=None, postfix=None, format='', default_value=None, condition=None, after_convert=None):
+  def __init__(self, prefix:Union[str,None]=None, postfix:Union[str,None]=None,
+                     format:str='', default_value:Any=None,
+                     condition:Union[Callable[[Any], Union[bool,str]],None]=None,
+                     after_convert:Union[Callable[Any, Any],None]=None):
+      """
+      Create the object.
+
+      Parameters
+      ----------
+      prefix
+        The string, that will be printed before the value
+
+      postfix
+        The string, that will be printed after the value
+
+      format
+        The (python) format string, that will be used for printing the value.
+        The format is passed as format argument to ``str.format`` routine.
+
+      default_value
+        The default value of the options of this type. ``None`` means no default value.
+
+      condition
+        Function, that check the validity of the value. It should return ``True`` for a valid
+        value, and ``False`` or string for invalid. The string is interpreted as an error message
+        that explains the invalidity of the value.
+
+      after_convert
+        Function, that - if it is given - is applied to the (entered or parsed) value. The function
+        is applied on the result of the
+        :meth:`convert<ase2sprkkr.common.grammar_types.BaseType.convert>` method
+      """
+
       self.prefix = prefix
+      """ The string, that will be printed before the value """
       self.postfix = postfix
+      """ The string, that will be printed after the value """
       self.format = format
+      """ The (python) format string, that will be used for printing the value.
+        The format is passed as format argument to ``str.format`` routine.  """
       self.condition = condition
       if after_convert is not None:
          self.convert = lambda v: \
@@ -206,6 +253,7 @@ class BaseType:
     pass
 
 class Unsigned(BaseType):
+  """ Unsigned integer (zero is possible) """
 
   _grammar = replace_whitechars(ppc.integer).setParseAction(lambda x:int(x[0]))
 
@@ -221,6 +269,7 @@ class Unsigned(BaseType):
 Unsigned.I = Unsigned()
 
 class Integer(BaseType):
+  """ Signed integer """
 
   _grammar = replace_whitechars(ppc.signed_integer).setParseAction(lambda x:int(x[0]))
 
@@ -235,7 +284,7 @@ class Integer(BaseType):
 Integer.I = Integer()
 
 class Bool(BaseType):
-
+  """ A bool type, whose value is represented by a letter (T or F) """
   _grammar = (pp.CaselessKeyword('T') | pp.CaselessKeyword('F')).setParseAction( lambda x: x[0] == 'T' )
 
   def _validate(self, value, parse_check=False):
@@ -253,7 +302,7 @@ Bool.I = Bool()
 
 
 class Real(BaseType):
-
+  """ A real value """
   _grammar = replace_whitechars(ppc.fnumber).setParseAction(lambda x: float(x[0]))
 
   def _validate(self, value, parse_check=False):
@@ -267,6 +316,7 @@ class Real(BaseType):
 Real.I = Real()
 
 class Date(BaseType):
+  """ A date value of the form 'DD.MM.YYYY' """
 
   _grammar = pp.Regex(r'(?P<d>\d{2}).(?P<m>\d{2}).(?P<y>\d{4})').setParseAction(lambda x: datetime.date(int(x['y']), int(x['m']), int(x['d'])))
 
@@ -283,8 +333,12 @@ Date.I = Date()
 
 
 class BaseRealWithUnits(BaseType):
+  """ The base class for float value, which can have units append.
+      The value is converted automatically to the base units.
+  """
 
   grammar_cache = {}
+  """ A grammar for units is cached """
 
   def _grammar_units(self, units):
     i = id(units)
@@ -311,22 +365,27 @@ class BaseRealWithUnits(BaseType):
   numpy_type = float
 
 class RealWithUnits(BaseRealWithUnits):
+  """ A float value with user-defined units """
 
   def __init__(self, *args, units, **kwargs):
      self.units = units
      super().__init__(*args, **kwargs)
 
 class Energy(BaseRealWithUnits):
+  """ The grammar type for energy. The default units are Rydberg, one can specify eV. """
 
   units = {
       'Ry' : 1.,
       'eV' : 1. / Rydberg,
       None : 1.,
   }
+  """ The allowed units and their conversion factors """
 
 Energy.I = Energy()
 
 class BaseString(BaseType):
+  """ Base type for string grammar types """
+
   def _validate(self, value, parse_check=False):
     if not isinstance(value, str): return "String value required"
     if not parse_check:
@@ -337,6 +396,7 @@ class BaseString(BaseType):
     return True
 
 class String(BaseString):
+  """ Just a string (without whitespaces and few special chars) """
   _grammar = Word(pp.printables,excludeChars=",;{}").setParseAction(lambda x:x[0])
 
   def grammar_name(self):
@@ -344,7 +404,7 @@ class String(BaseString):
 String.I = String()
 
 class QString(BaseString):
-  """ Quoted string"""
+  """ Either a quoted string, or just a word (without whitespaces or special chars) """
   _grammar = (pp.Word(pp.printables, excludeChars=",;{}") or pp.QuotedString("'")).setParseAction(lambda x:x[0])
 
   def grammar_name(self):
@@ -353,7 +413,7 @@ class QString(BaseString):
 QString.I = String()
 
 class LineString(BaseString):
-  """ String up to the end of line """
+  """ A string, that takes all up to the end of the line """
   _grammar = pp.SkipTo(pp.LineEnd() | pp.StringEnd())
 
   def grammar_name(self):
@@ -362,11 +422,15 @@ class LineString(BaseString):
 LineString.I = LineString()
 
 class Keyword(BaseType):
+  """
+  A value, that can take values from the predefined set of strings.
+  """
 
   def __init__(self, *keywords, **kwargs):
     super().__init__(**kwargs)
     self.keywords = [ i.upper() for i in keywords ]
-    self._grammar = optional_quote + pp.MatchFirst((pp.CaselessKeyword(i) for i in self.keywords)).setParseAction(lambda x: x[0].upper()) + optional_quote
+    with generate_grammar():
+      self._grammar = optional_quote + pp.MatchFirst((pp.CaselessKeyword(i) for i in self.keywords)).setParseAction(lambda x: x[0].upper()) + optional_quote
 
   def _validate(self, value, parse_check=False):
     return value in self.keywords or "Required one of [" + "|".join(self.keywords) + "]"
@@ -381,10 +445,16 @@ class Keyword(BaseType):
       return value.upper()
 
 def DefKeyword(default, *others, **kwargs):
+  """
+  A value, that can take values from the predefined set of strings, the first one is the default value.
+  """
   return Keyword(default, *others, default_value=default, **kwargs)
 
 
 class Flag(BaseType):
+  """
+  A boolean value, which is True, if a name of the value appears in the input file.
+  """
 
   def grammar_name(self):
       return None
@@ -405,10 +475,19 @@ Flag.I = Flag()
 normalize_type_map = {
     np.int64 : int,
     np.float64: float,
-    np.bool: bool
+    np.bool_: bool
 }
+""" Mapping of alternative types to the 'canonical ones'. """
 
 def normalize_type(type):
+    """ Return the 'canonical type' for a given type
+
+    I.e. it maps numpy internal types to standard python ones
+
+    doctest:
+    >>> normalize_type(np.int64)
+    int
+    """
     return normalize_type_map.get(type, type)
 
 type_from_type_map = OrderedDict([
@@ -417,6 +496,11 @@ type_from_type_map = OrderedDict([
     (bool,  Bool.I),
     (str  , String.I)]
 )
+""" The standard grammar_types for python types.
+
+The value type can be given by a standard python type, this map maps the
+python type for the appropriate grammar_type class.
+"""
 
 def format_for_type(format, type):
   """
@@ -442,7 +526,7 @@ def type_from_type(type, format='', format_all=False):
       Parameters
       ----------
       type: A python type or BaseType
-        A type to be converted to a grammar element (BaseType class descendant)
+        A type to be converted to a grammar type (BaseType class descendant)
 
       format: str or dict
         The format to be applied to the resulting class. If dict is given, see 'format_for_type'
@@ -467,6 +551,7 @@ def type_from_type(type, format='', format_all=False):
 
 
 class Array(BaseType):
+  """ A (numpy) array of values of one type """
 
   delimiter=pp.White(' \t').suppress()
   delimiter_str = ' '
@@ -569,17 +654,25 @@ class SetOf(Array):
   def __str__(self):
     return "SetOf({})".format(str(self.type))
 
-
-
-""" Map python native types to configuration value types """
-
 type_from_set_map = OrderedDict([
     (float, SetOf(float)),
     (int  , SetOf(int)),
 ])
+""" Map the python type of a collection member to a grammar type of the collection.
+
+Only canonical types are expected, see :meth:`ase2sprkkr.common.grammar_types.normalize_type`
+"""
 
 def type_from_value(value):
-  """ Gues the option type from a python value. E.g. 2 => Integer """
+  """ Gues the grammar type from a python value.
+
+  ..doctest::
+  >>> type_from_value(2)
+  Integer
+
+  >>> type_from_value(2.0)
+  Real
+  """
   if isinstance(value, (list, np.ndarray)):
      return type_from_set_map[normalize_type(value[0].__class__)] if len(value) else Integer.I
   if isinstance(value, str):
@@ -594,6 +687,14 @@ def type_from_value(value):
   return type.__class__(default_value = value)
 
 def type_from_default_value(value, format='', format_all=False):
+   """ Guess the grammar type from a value, that will become the default value of the grammar type.
+
+   It has to create a new object instance, as it has to set the default
+   value property of the returned object. An (output) format can be applied to the
+   resulting grammar type
+
+   Grammar types passed as types are left as is, unless format_all flag is set.
+   """
    if inspect.isclass(value) or isinstance(value, BaseType):
       return type_from_type(value, format=format, format_all=format_all)
    ptype = normalize_type(value.__class__)
@@ -601,6 +702,15 @@ def type_from_default_value(value, format='', format_all=False):
    return gtype(default_value = value, format=format_for_type(format, ptype))
 
 class BaseMixed(BaseType):
+  """
+  A variant type - it can hold "anything".
+  """
+
+  type = None
+  """ The types, that the value can hold. To be redefined in the descendants. """
+
+  string_type = None
+  """ Type of string grammar_type to be used.  To be redefined in the descendants. """
 
   @classmethod
   def _grammar(cls, param_name=False):
@@ -609,6 +719,7 @@ class BaseMixed(BaseType):
     ))
 
   def get_type(self, value):
+      """ Return the type of the value """
       return self.string_type if isinstance(value, str) else type_from_value(value)
 
   def _validate(self, value, parse_check=False):
@@ -622,8 +733,10 @@ class BaseMixed(BaseType):
 
 
 class Mixed(BaseMixed):
+  """ A variant value to be used in input files (in unknown - custom - options) """
 
   string_type = QString.I
+  """ Input files use quoted strings. """
 
   types = [
       Energy.I,
@@ -641,8 +754,10 @@ class Mixed(BaseMixed):
 Mixed.I = Mixed()
 
 class PotMixed(BaseMixed):
+  """ A variant value to be used in potential files (in unknown - custom - options) """
 
   string_type = LineString.I
+  """ Potential files use line strings. """
 
   types = [
       Energy.I,
@@ -664,7 +779,7 @@ PotMixed.I = PotMixed()
 
 
 class Separator(BaseType):
-  """ Special class for **** separator inside a section """
+  """ Special class for ``****`` separator inside a section """
 
   _grammar = separator_grammar.copy().setParseAction(lambda x: [None])
   has_value = False
@@ -770,10 +885,15 @@ class Sequence(BaseType):
 
 
 class Table(BaseType):
-  """ Table, optionaly with named columns, e.g.
+  """
+  Table, optionaly with named columns, e.g.
+
+    ::text
+
       IQ     IREFQ       IMQ       NOQ  ITOQ  CONC
        1         1         1         1     1 1.000
        2         2         2         1     2 1.000
+
   """
 
   name_in_grammar = False
@@ -936,19 +1056,29 @@ class Table(BaseType):
       return f"<TABLE of {data}>"
 
 integer = Integer.I
+""" A standard signed integer grammar type instance """
 unsigned = Unsigned.I
+""" A standard unsigned integer grammar type instance """
 boolean = Bool.I
+""" A standard bool grammar type instance (for potential files) """
 flag = Flag.I
+""" A standard bool grammar type instance (for input files) """
 real = Real.I
+""" A standard real grammar type instance """
 string = String.I
+""" A standard string grammar type instance """
 qstring = QString.I
+""" A standard quoted string grammar type instance (for input files) """
 line_string = LineString.I
+""" A standard line string grammar type instance (for potential files) """
 mixed = Mixed.I
+""" A standard variant grammar type instance (for input files) """
 pot_mixed = PotMixed.I
+""" A standard variant grammar type instance (for potential files) """
 separator = Separator.I
+""" A standard separator line grammar type instance (for potential files) """
 energy = Energy.I
-
-
+""" A standard energy float value type instance (for potential files) """
 
 context.__exit__(None, None, None)
 del context
