@@ -1,3 +1,8 @@
+""" Readers for processes outputs.
+
+They employ asyncio to makes possible read stdio and stderr concurently.
+"""
+
 import asyncio
 import functools
 import subprocess
@@ -6,9 +11,11 @@ import numpy as np
 
 class BaseProcessOutputReader:
   """
-  Class, that run a process, optionally saves all the output of the process to the file
+  Class, that run a process, optionally saves all the output of the process to a file,
   and pass the stdout and stderr of the process to its two asyn routines, read_error
-  and read_output
+  and read_output.
+
+  The descendant can redefine the routines to parse the output (or its parts).
   """
   async def run_subprocess(self):
 
@@ -61,12 +68,36 @@ class BaseProcessOutputReader:
       return self.result(*out)
 
   def result(self, output, error, wait):
-      """ """
+      """ This function is for postprocessing the results.
+
+      It is intended to be predefined in the descendants
+
+      Parameters
+      ----------
+
+      output: mixed
+        Result of the self.read_output
+
+      error: mixed
+        Result of the self.read_error
+
+      wait: int
+        The process return value
+
+      Return
+      ------
+
+      out: mixed
+        Currently, the tuple (output, error) is returned, however,
+        subclasses can return anything they want.
+      """
+
       if wait != 0:
-         raise ValueError("The process ended with return value {wait}")
-      if error is not None:
-         return output, error
-      return output
+         e = ValueError("The process ended with return value {wait}")
+         e.output = output
+         e.error = error
+         raise e
+      return output, error
 
   def run(self, cmd, outfile, print_output=False, directory=None, **kwargs):
       self.cmd = cmd
@@ -114,8 +145,9 @@ class BaseProcessOutputReader:
 
 
 class AsyncioFileReader:
-  """ `Async' file reader that mimics asyncio StreamReader
-      It is in fact not async, but offers the same interface.
+  """ File reader that mimics asyncio StreamReader.
+      In fact, it is synchronous, but it offers the
+      same interface as the asyncio StreamReader.
   """
 
   def __init__(self, filename, buffersize=8192):
